import { IResponseBase } from "../Response";

/**
 * Cloud services status
 */
enum CloudServiceStatus
{
	SUCCESS = "success",
	PENDING = "pending",
	FAILED = "failed",
	NOT_STARTED = "not_started",
	NOT_EXIST = "nonexistent",
	UPLOADING = "uploading",
	PROCESSING = "processing",
}

interface IJob
{
	/**
	 * Id of the job
	 */
	jobId?: number;

	/**
	 * Id of service this job is related to
	 */
	serviceId: number;

	/**
	 * Name of the job
	 */
	jobName: string;

	/**
	 * Status of the job
	 */
	status: CloudServiceStatus;

	/**
	 * Current progress of the job, in percentage
	 */
	percentage: number;

	/**
	 * Result of the job
	 */
	result?: string;

	/**
	 * Timestamp for creation time of this mission
	 */
	createdAt?: number;

	/**
	 * other prop
	 */
	serviceInfo?: {
		/**
		 * URL of Icon that save in CDN of this mission
		 */
		iconUrl?: string;
	};

	/**
	 * Feature hashes related to this job
	 */
	featureHashList?: string[];

	/**
	 * Description ot the job
	 */
	description?: string;
}

interface ICreateCloudJobParams
{
	/**
	 * Access token to validate user
	 */
	accessToken: string;

	/**
	 * Id of service we are using to start the job
	 */
	serviceId: number;

	/**
	 * Name of the job
	 */
	jobName: string;
}

interface ICreateCloudJobResponse extends IResponseBase
{
	data: IJob;
}

interface IAddCloudJobResourcesParams
{
	/**
	 * Access token to validate user
	 */
	accessToken: string;

	/**
	 * Id of job we are updating resources for
	 */
	jobId: number;
}

interface IStartCloudJobParams
{
	/**
	 * Access token to validate user
	 */
	accessToken: string;

	/**
	 * Id of job we are starting
	 */
	jobId: number;
}

interface IStartCloudJobResponse extends IResponseBase
{
	data: IJob;
}

interface ICreateMeshJobParams
{
	/**
	 * Access token to validate user
	 */
	accessToken: string;

	/**
	 * Id of service we are using to start the job
	 */
	serviceId: number;

	/**
	 * Name of the job
	 */
	jobName: string;

	/**
	 * Id of a mesh mission this job is related to
	 */
	missionId: number;

	/**
	 * List of feature hash related to this job
	 */
	featureHashList: string[];
}

interface ICreateMeshJobResponse extends IResponseBase
{
	data: IJob;
}

interface IGetCloudJobsParams
{
	/**
	 * Access token to validate user
	 */
	accessToken?: string;

	/**
	 * Pagination parameter
	 */
	page: number;

	/**
	 * Pagination parameter
	 */
	count: number;

	/**
	 * Temporal ordering
	 */
	order?: string;
}

interface IGetCloudJobsResponse extends IResponseBase
{
	data: {
		/**
		 * Whether we have more jobs unloaded
		 */
		more: boolean;

		jobs: IJob[];
	}
}

interface IGetMeshJobsParams
{
	/**
	 * Access token to validate user
	 */
	accessToken: string;

	/**
	 * Id of mission this service is related to
	 */
	missionId: number;
}

interface IGetMeshJobsResponse extends IResponseBase
{
	data: {

		jobs: IJob[];
	}
}

interface IGetJobStatusParams
{
	/**
	 * Access token to validate user
	 */
	accessToken: string;

	/**
	 * Ids of jobs we are interested
	 */
	jobIdList: number[];
}

interface IGetJobStatusResponse extends IResponseBase
{
	data: {
		jobs: IJob[],
	}
}

interface IRemoveJobsParams
{
	/**
	 * Access token to validate user
	 */
	accessToken: string;

	/**
	 * Ids of jobs we are removing
	 */
	jobIdList: number[];
}

interface IRemoveJobsResponse extends IResponseBase
{
	data: IJob[]
}

interface ISearchJobsParams
{
	/**
	 * Access token to validate user
	 */
	accessToken: string;

	/**
	 * Job name we are interested in (i.e. searching)
	 */
	jobName: string;

	/**
	 * Optional, id of service the job must be related to
	 */
	serviceId?: number;

	/**
	 * Optional, resource url set
	 */
	resources?: string[];
}

interface ISearchJobsResponse extends IResponseBase
{
	data: {
		jobs: IJob[],
	},
}

interface IRestartJobsParams
{
	/**
	 * Access token to validate user
	 */
	accessToken: string;

	/**
	 * Ids of jobs we are restarting
	 */
	jobIdList: number[];
}

interface IRestartJobsResponse extends IResponseBase
{
	data: IJob[]
}


interface IBulkCreateMeshJobParams
{
	/**
	 * Access token to validate user
	 */
	accessToken: string;

	/**
	 * Id of service we are using to start the job
	 */
	serviceId: number;

	/**
	 * Id of a mesh mission this job is related to
	 */
	missionId: number;

	/**
	 * create job according to feature in this list
	 */
	featureList: IFeatureJob[];
}

interface IFeatureJob
{

	/**
	 * The feature hash
	 */
	featureHash: string;

	/**
	 * Name of the job
	 */
	jobName: string;
}

interface IBulkCreateMeshJobResponse extends IResponseBase
{
	data: IJob[];
}

export
{
	IJob,
	CloudServiceStatus,
	ICreateCloudJobParams,
	ICreateCloudJobResponse,
	ICreateMeshJobParams,
	ICreateMeshJobResponse,
	IGetCloudJobsParams,
	IGetCloudJobsResponse,
	IGetMeshJobsParams,
	IGetMeshJobsResponse,
	IGetJobStatusParams,
	IGetJobStatusResponse,
	IRestartJobsParams,
	IRestartJobsResponse,
	IRemoveJobsParams,
	IRemoveJobsResponse,
	ISearchJobsParams,
	ISearchJobsResponse,
	IAddCloudJobResourcesParams,
	IStartCloudJobParams,
	IStartCloudJobResponse,
	IBulkCreateMeshJobParams,
	IBulkCreateMeshJobResponse,
};
